==> allensdk-2.13.0-py3-none-any.whl: allensdk/brain_observatory/behavior/data_objects/metadata/behavior_metadata/date_of_acquisition.py <==
import warnings
from datetime import datetime

import pytz
from pynwb import NWBFile

from allensdk.brain_observatory.behavior.data_files import StimulusFile
from allensdk.brain_observatory.behavior.data_objects import DataObject
from allensdk.brain_observatory.behavior.data_objects.base \
    .readable_interfaces import \
    JsonReadableInterface, LimsReadableInterface, NwbReadableInterface
from allensdk.internal.api import PostgresQueryMixin


class DateOfAcquisition(DataObject, LimsReadableInterface,
                        JsonReadableInterface, NwbReadableInterface):
    """timestamp for when experiment was started in UTC"""
    def __init__(self, date_of_acquisition: datetime):
        super().__init__(name="date_of_acquisition", value=date_of_acquisition)

    @classmethod
    def from_json(cls, dict_repr: dict) -> "DateOfAcquisition":
        doa = dict_repr['date_of_acquisition']
        doa = datetime.strptime(doa, "%Y-%m-%d %H:%M:%S")
        tz = pytz.timezone("America/Los_Angeles")
        doa = tz.localize(doa)

        # NOTE: LIMS writes to JSON in local time. Needs to be converted to UTC
        doa = doa.astimezone(pytz.utc)

        return cls(date_of_acquisition=doa)

    @classmethod
    def from_lims(
            cls, behavior_session_id: int,
            lims_db: PostgresQueryMixin) -> "DateOfAcquisition":
        query = """
                SELECT bs.date_of_acquisition
                FROM behavior_sessions bs
                WHERE bs.id = {};
                """.format(behavior_session_id)

        experiment_date = lims_db.fetchone(query, strict=True)
        experiment_date = cls._postprocess_lims_datetime(
            datetime=experiment_date)
        return cls(date_of_acquisition=experiment_date)

    @classmethod
    def from_nwb(cls, nwbfile: NWBFile) -> "DateOfAcquisition":
        return cls(date_of_acquisition=nwbfile.session_start_time)

    def validate(self, stimulus_file: StimulusFile,
                 behavior_session_id: int) -> "DateOfAcquisition":
        """raise a warning if the date differs too much from the
        datetime obtained from the behavior stimulus (*.pkl) file."""
        pkl_data = stimulus_file.data
        pkl_raw_acq_date = pkl_data["start_time"]
        if isinstance(pkl_raw_acq_date, datetime):
            pkl_acq_date = pytz.utc.localize(pkl_raw_acq_date)

        elif isinstance(pkl_raw_acq_date, (int, float)):
            # We are dealing with an older pkl file where the acq time is
            # stored as a Unix style timestamp string
            parsed_pkl_acq_date = datetime.fromtimestamp(pkl_raw_acq_date)
            pkl_acq_date = pytz.utc.localize(parsed_pkl_acq_date)
        else:
            pkl_acq_date = None
            warnings.warn(
                "Could not parse the acquisition datetime "
                f"({pkl_raw_acq_date}) found in the following stimulus *.pkl: "
                f"{stimulus_file.filepath}"
            )

        if pkl_acq_date:
            acq_start_diff = (
                    self.value - pkl_acq_date).total_seconds()
            # If acquisition dates differ by more than an hour
            if abs(acq_start_diff) > 3600:
                session_id = behavior_session_id
                warnings.warn(
                    "The `date_of_acquisition` field in LIMS "
                    f"({self.value}) for behavior session "
                    f"({session_id}) deviates by more "
                    f"than an hour from the `start_time` ({pkl_acq_date}) "
                    "specified in the associated stimulus *.pkl file: "
                    f"{stimulus_file.filepath}"
                )
        return self

    @staticmethod
    def _postprocess_lims_datetime(datetime: datetime):
        """Applies postprocessing to datetime read from LIMS"""
        # add utc tz
        datetime = pytz.utc.localize(datetime)

        return datetime


class DateOfAcquisitionOphys(DateOfAcquisition):
    """Ophys experiments read date of acquisition from the ophys_sessions
    table in LIMS instead of the behavior_sessions table"""

    @classmethod
    def from_lims(
            cls, ophys_experiment_id: int,
            lims_db: PostgresQueryMixin) -> "DateOfAcquisitionOphys":
        query = f"""
            SELECT os.date_of_acquisition
            FROM ophys_experiments oe
            JOIN ophys_sessions os ON oe.ophys_session_id = os.id
            WHERE oe.id = {ophys_experiment_id};
        """
        doa = lims_db.fetchone(query=query)
        doa = cls._postprocess_lims_datetime(
            datetime=doa)
        return DateOfAcquisitionOphys(date_of_acquisition=doa)

==> allensdk-2.13.6-py3-none-any.whl: allensdk/brain_observatory/behavior/data_objects/metadata/behavior_metadata/date_of_acquisition.py <==
import warnings
from datetime import datetime

import pytz
from pynwb import NWBFile

from allensdk.brain_observatory.behavior.data_files import BehaviorStimulusFile
from allensdk.core import DataObject
from allensdk.core import \
    JsonReadableInterface, LimsReadableInterface, NwbReadableInterface
from allensdk.internal.api import PostgresQueryMixin


class DateOfAcquisition(DataObject, LimsReadableInterface,
                        JsonReadableInterface, NwbReadableInterface):
    """timestamp for when experiment was started in UTC"""
    def __init__(self, date_of_acquisition: datetime):
        super().__init__(name="date_of_acquisition", value=date_of_acquisition)

    @classmethod
    def from_json(cls, dict_repr: dict) -> "DateOfAcquisition":
        doa = dict_repr['date_of_acquisition']
        doa = datetime.strptime(doa, "%Y-%m-%d %H:%M:%S")
        tz = pytz.timezone("America/Los_Angeles")
        doa = tz.localize(doa)

        # NOTE: LIMS writes to JSON in local time. Needs to be converted to UTC
        doa = doa.astimezone(pytz.utc)

        return cls(date_of_acquisition=doa)

    @classmethod
    def from_lims(
            cls, behavior_session_id: int,
            lims_db: PostgresQueryMixin) -> "DateOfAcquisition":
        query = """
                SELECT bs.date_of_acquisition
                FROM behavior_sessions bs
                WHERE bs.id = {};
                """.format(behavior_session_id)

        experiment_date = lims_db.fetchone(query, strict=True)
        experiment_date = cls._postprocess_lims_datetime(
            datetime=experiment_date)
        return cls(date_of_acquisition=experiment_date)

    @classmethod
    def from_nwb(cls, nwbfile: NWBFile) -> "DateOfAcquisition":
        return cls(date_of_acquisition=nwbfile.session_start_time)

    def validate(self, stimulus_file: BehaviorStimulusFile,
                 behavior_session_id: int) -> "DateOfAcquisition":
        """raise a warning if the date differs too much from the
        datetime obtained from the behavior stimulus (*.pkl) file."""
        pkl_data = stimulus_file.data
        pkl_raw_acq_date = pkl_data["start_time"]
        if isinstance(pkl_raw_acq_date, datetime):
            pkl_acq_date = pytz.utc.localize(pkl_raw_acq_date)

        elif isinstance(pkl_raw_acq_date, (int, float)):
            # We are dealing with an older pkl file where the acq time is
            # stored as a Unix style timestamp string
            parsed_pkl_acq_date = datetime.fromtimestamp(pkl_raw_acq_date)
            pkl_acq_date = pytz.utc.localize(parsed_pkl_acq_date)
        else:
            pkl_acq_date = None
            warnings.warn(
                "Could not parse the acquisition datetime "
                f"({pkl_raw_acq_date}) found in the following stimulus *.pkl: "
                f"{stimulus_file.filepath}"
            )

        if pkl_acq_date:
            acq_start_diff = (
                    self.value - pkl_acq_date).total_seconds()
            # If acquisition dates differ by more than an hour
            if abs(acq_start_diff) > 3600:
                session_id = behavior_session_id
                warnings.warn(
                    "The `date_of_acquisition` field in LIMS "
                    f"({self.value}) for behavior session "
                    f"({session_id}) deviates by more "
                    f"than an hour from the `start_time` ({pkl_acq_date}) "
                    "specified in the associated stimulus *.pkl file: "
                    f"{stimulus_file.filepath}"
                )
        return self

    @staticmethod
    def _postprocess_lims_datetime(datetime: datetime):
        """Applies postprocessing to datetime read from LIMS"""
        # add utc tz
        datetime = pytz.utc.localize(datetime)

        return datetime


class DateOfAcquisitionOphys(DateOfAcquisition):
    """Ophys experiments read date of acquisition from the ophys_sessions
    table in LIMS instead of the behavior_sessions table"""

    @classmethod
    def from_lims(
            cls, ophys_experiment_id: int,
            lims_db: PostgresQueryMixin) -> "DateOfAcquisitionOphys":
        query = f"""
            SELECT os.date_of_acquisition
            FROM ophys_experiments oe
            JOIN ophys_sessions os ON oe.ophys_session_id = os.id
            WHERE oe.id = {ophys_experiment_id};
        """
        doa = lims_db.fetchone(query=query)
        doa = cls._postprocess_lims_datetime(
            datetime=doa)
        return DateOfAcquisitionOphys(date_of_acquisition=doa)
